#include "GLine.h"

GLine::GLine(void)
{
	xx=CComplex(0.0,0.0);
	yy=CComplex(0.0,0.0);
	zz=CComplex(1.0,0.0);
	clipping=false;
	next=NULL;
}

GLine::GLine(CComplex x,CComplex y,CComplex z)
{
	xx=x;
	yy=y;
	zz=z;
	clipping=false;
	next=NULL;
}

GLine::GLine(double x1, double y1, double x2, double y2, int rr, int gg, int bb, int radius)
{
	if(x1==x2 && y1==y2){
		xx=CComplex(0.0,0.0);
		yy=CComplex(0.0,0.0);
		zz=CComplex(1.0,0.0);
	}
	else {
		xx=CComplex(y1-y2,0.0);
		yy=CComplex(x2-x1,0.0);
		zz=CComplex(x1*y2-x2*y1,0.0);
	}
	clipping=false;
	next=NULL;
}

GLine::GLine(const GLine& p)
{
	xx=p.xx;
	yy=p.yy;
	zz=p.zz;
	clipping=p.clipping;
	next=NULL;
}


GLine::~GLine(void)
{
}


bool GLine::realp(double* rx,double* ry,double* rz)
{
	if(zz.zerop()){
		if(yy.zerop()){
			*rx=1.0;*ry=0.0;*rz=0.0;
			return true;
		}
		else {
			*rx=xy().real();
			*ry=1.0;
			*rz=0.0;
			return xy().realp();
		}
	}
	else {
		*rx=xz().real();
		*ry=yz().real();
		*rz=1.0;
		return xz().realp()  && yz().realp();
	}
}

CComplex GLine::xy(void)
{
	return xx/yy;
}

CComplex GLine::xz(void)
{
	return xx/zz;
}

CComplex GLine::yz(void)
{
	return yy/zz;
}

bool GLine::SetLine(double x1, double y1, double x2, double y2)
{
	if(x1==x2 && y1==y2){
		xx=CComplex(0.0,0.0);
		yy=CComplex(0.0,0.0);
		zz=CComplex(1.0,0.0);
	}
	else {
		xx=CComplex(y1-y2,0.0);
		yy=CComplex(x2-x1,0.0);
		zz=CComplex(x1*y2-x2*y1,0.0);
	}
	return false;
}

int GLine::NumOfLn(void)
{
	if(next==NULL){
		return 1;
	}
	else {
		return (next->NumOfLn() + 1);
	}
}

#include "wx/wx.h"
#include "KidsCindyCanvas.h"
#include "resource.h"

GLine* MyCanvas::NewLine(double x1, double y1, double x2, double y2)
{
	int i;
	for(i=0;i<NUM_OF_POINTS;i++){
		if(linebuffer[i].used==false){
			if(x1==x2 && y1==y2){
				linebuffer[i].xx=CComplex(0.0,0.0);
				linebuffer[i].yy=CComplex(0.0,0.0);
				linebuffer[i].zz=CComplex(1.0,0.0);
			}
			else {
				linebuffer[i].xx=CComplex(y1-y2,0.0);
				linebuffer[i].yy=CComplex(x2-x1,0.0);
				linebuffer[i].zz=CComplex(x1*y2-x2*y1,0.0);
			}
			linebuffer[i].next=NULL;
			linebuffer[i].used=true;
			linebuffer[i].vanishing=0;
			linebuffer[i].ics=NULL;
			linebuffer[i].last_ic=&(linebuffer[i].ics);
			linebuffer[i].NOics=0;
			linebuffer[i].invisible=false;
			linebuffer[i].lndsn=LineMode-LN_1;
			linebuffer[i].offsetx=0;
			linebuffer[i].offsety=0;
			linebuffer[i].expand=1.0;
			linebuffer[i].interval=0.2;
			return &(linebuffer[i]);
		}
	}
	return NULL;
}

GLine* MyCanvas::NewLine(CComplex xxx, CComplex yyy, CComplex zzz)
{
	int i;
	for(i=0;i<NUM_OF_POINTS;i++){
		if(linebuffer[i].used==false){
			linebuffer[i].xx=xxx;
			linebuffer[i].yy=yyy;
			linebuffer[i].zz=zzz;
			linebuffer[i].next=NULL;
			linebuffer[i].used=true;
			linebuffer[i].vanishing=0;
			linebuffer[i].ics=NULL;
			linebuffer[i].last_ic=&(linebuffer[i].ics);
			linebuffer[i].NOics=0;
			linebuffer[i].invisible=false;
			linebuffer[i].lndsn=LineMode-LN_1;
			linebuffer[i].offsetx=0;
			linebuffer[i].offsety=0;
			linebuffer[i].expand=1.0;
			linebuffer[i].interval=0.2;
			return &(linebuffer[i]);
		}
	}
	return NULL;
}
void MyCanvas::AssignLine(GLine* ln)
{
	*lastln = ln;
	lastln= &(ln->next);
	sheetbuffer[currentSheet].MyLine=lns;
	sheetbuffer[currentSheet].MyLastLine=lastln;
}

//bool MyCanvas::DeleteLine(GLine* l)
//{
//	l->used=false;
//	return false;
//}
bool MyCanvas::DeleteAllLines(GLine* l)
{
	if(l==NULL){
		return false;
	}
	DeleteAllLines(l->next);
	l->used=false;
	return false;
}
GLine* MyCanvas::makeLClone(GLine* l) // ܂
{
	if(l!=NULL){
		int i;
		for(i=0;i<NUM_OF_POINTS;i++){
			if(linebuffer[i].used==false){
				GLine* lnp=&(linebuffer[i]);
				lnp->xx=l->xx;
				lnp->yy=l->yy;
				lnp->zz=l->zz;
				lnp->next=NULL;
				lnp->used=true;
				lnp->ics=NULL;
				lnp->last_ic=&(lnp->ics);
				lnp->NOics=-999;  // UNDEFINITE CODE
				lnp->lndsn=l->lndsn;
				lnp->vanishing = l->vanishing + 1; 
				lnp->clipping = l->clipping;
				lnp->afterimage_x1=l->afterimage_x1;
				lnp->afterimage_y1=l->afterimage_y1;
				lnp->afterimage_x2=l->afterimage_x2;
				lnp->afterimage_y2=l->afterimage_y2;
				lnp->invisible=l->invisible;
				lnp->expand = l->expand;
				if(l->next!=NULL){
					lnp->next=makeLClone(l->next);
				}
				else {
					lnp->next=NULL;
				}
				return lnp;
			}
		}
	}
	return NULL;
}


bool MyCanvas::copyToLClone(GLine* tgt,GLine*src) // N[Ƀf[^Rs[
{
	// tgt->*** = src-> ***;// F𔖂ăRs[B炱N[B
	if(tgt==NULL || src==NULL){
		return false;
	}
	tgt->xx=src->xx;
	tgt->yy=src->yy;
	tgt->zz=src->zz;
	tgt->vanishing = src->vanishing + 1; 
	tgt->clipping = src->clipping;
	tgt->afterimage_x1=src->afterimage_x1;
	tgt->afterimage_y1=src->afterimage_y1;
	tgt->afterimage_x2=src->afterimage_x2;
	tgt->afterimage_y2=src->afterimage_y2;
	tgt->invisible=src->invisible;
	tgt->expand = src->expand;
	if(tgt->next!=NULL && src->next != NULL){
		return copyToLClone(tgt->next,src->next);
	}
	return false;
}

bool MyCanvas::deleteLClone(GLine* l) // N[
{
	DeleteAllLines(l);
	return false;
}
