//
//
//

#include "Meet.h"
#include <math.h>

GPoint Meet(GLine* la, GLine* lb)
{
	return GPoint(la->yy * lb->zz - la->zz * lb->yy,
		la->zz * lb->xx - la->xx * lb->zz,
		la->xx * lb->yy - la->yy * lb->xx);
}

void Meet(GCircle* ca, GCircle* cb,double *gx,double *gy)
{
	for(int i=0;i<3;i++){
		Proj2Circle(ca,gx,gy);
		Proj2Circle(cb,gx,gy);
	}
}

void Proj2Circle(GCircle* ca, double* gx,double *gy)
{
	double a,b,r;
	if(ca->realp(&a,&b,&r)==false){
		*gx = ca->xx.real()+sqrt(ca->rr.real());
		*gy = (ca->yy).real();
		return;
	}
	double vx=*gx-a;
	double vy=*gy-b;
	double vr=sqrt(vx*vx+vy*vy);
	if(vr==0){
		*gx = a+sqrt(r);
		*gy = b;
		return;
	}
	double ratio=sqrt(r)/vr;
	*gx = a+vx*ratio;
	*gy = b+vy*ratio;
	return ;
}

void Proj2Circle(GCircle* ca, CComplex* gx, CComplex *gy)
{
	CComplex vx=*gx - ca->xx;
	CComplex vy=*gy - ca->yy;
	CComplex vr=sqrtC(vx*vx+vy*vy);
	if(vr.real()==0 && vr.imag()==0){
		*gx = ca->xx + sqrtC(ca->rr);
		*gy = ca->yy;
		return;
	}
	CComplex ratio=sqrtC(ca->rr)/vr;
	*gx = ca->xx + vx*ratio;
	*gy = ca->yy + vy*ratio;
}

void Proj2Line(GLine* la, CComplex* gx, CComplex *gy)
{
	CComplex a = la->xx;
	CComplex b = la->yy;
	CComplex c = la->zz;
	CComplex xxx1 =  (*gx) * (b * b - a * a) - CComplex(2.0,0.0) * (*gy) * a * b - CComplex(2.0,0.0) * a * c;
	CComplex xxx2 =  (*gy) * (a * a - b * b) - CComplex(2.0,0.0) * (*gx) * a * b - CComplex(2.0,0.0) * b * c;
	CComplex yyy = a * a + b * b;
	*gx = xxx1 / yyy;
	*gy = xxx2 / yyy;
}

void Meet_CL(CComplex aa,CComplex bb,CComplex cc, CComplex pp,CComplex qq,CComplex rr,CComplex *xx1 ,CComplex *xx2)
{
	CComplex c2=aa * aa + bb * bb;
	CComplex c1=(pp * bb * bb - aa * cc - aa * bb * qq);
	CComplex c0=bb * bb * pp * pp + (cc + bb * qq)*(cc + bb * qq) - bb * bb * rr;
	*xx1= (c1 + sqrtC(c1*c1-c2*c0)) /c2;
	*xx2= (c1 - sqrtC(c1*c1-c2*c0)) /c2;
}

#include "wx/wx.h"
#include "KidsCindyCanvas.h"

bool MyCanvas::incident(GLine* ln, GPoint* pt)
{
	CComplex dist=ln->xx * pt->xx + ln->yy * pt->yy + ln->zz * pt->zz;
	double distdenom=ln->xx.real()*ln->xx.real()+ln->yy.real()*ln->yy.real();
	if((dist.re * dist.re + dist.im * dist.im)/distdenom < 0.04){
		return true;
	}
	return false;
}

bool MyCanvas::incident(GCircle* ci, GPoint* pt)
{
	CComplex dist=(ci->xx - pt->xx) * (ci->xx - pt->xx) + (ci->yy - pt->yy) * (ci->yy - pt->yy) - ci->rr;
	if(dist.re * dist.re + dist.im * dist.im < 0.09){
		return true;
	}
	return false;
}

