//
//   Snapping.cpp
//
//
#include <math.h>
#include "wx/wx.h"
#include "KidsCindyCanvas.h"
#include "resource.h"
#include "Meet.h"

bool MyCanvas::Snapping(GPoint* p, double* gx,double* gy,GPoint** pt)
{
	if(p==NULL){
		*pt=NULL;
		return false;
	}
	double geomx=p->xz().real();
	double geomy=p->yz().real();
	if((geomx - *gx)*(geomx - *gx) + (geomy - *gy)*(geomy - *gy) < snapradius2 && p->invisible==false){
		*gx=geomx;
		*gy=geomy;
		*pt=p;
		return true;
	}
	if(p->next!=NULL){
		return Snapping(p->next,gx,gy,pt);
	}
	*pt=NULL;
	return false;
}

bool MyCanvas::NearPoint(GPoint* p, double* gx,double* gy,GPoint** pt)
{
	if(p==NULL){
		*pt=NULL;
		return false;
	}
	double geomx=p->xz().real();
	double geomy=p->yz().real();
	if((geomx - *gx)*(geomx - *gx) + (geomy - *gy)*(geomy - *gy) < snapradius2 && p->invisible==false){
		*pt=p;
		return true;
	}
	if(p->next!=NULL){
		return NearPoint(p->next,gx,gy,pt);
	}
	*pt=NULL;
	return false;
}

bool MyCanvas::Snap2Line(GLine* lines, double* gx, double* gy)
{
	if(lines==NULL){
		return false;
	}
	double a,b,c;
	if(lines->realp(&a,&b,&c)==false){
		return false;
	}
	double x0=-(a*c-b*(b*(*gx)-a*(*gy)))/(a*a+b*b);
	double y0=-(a*(b*(*gx)-a*(*gy))+b*c)/(a*a+b*b);
	if((x0- *gx)*(x0 - *gx)+(y0 - *gy)*(y0 - *gy) < snapradius2  && lines->invisible==false){
		*gx=x0;
		*gy=y0;
		return true;
	}
	if(lines->next!=NULL){
		return Snap2Line(lines->next,gx,gy);
	}
	return false;
}

int MyCanvas::Snap2LineEx(GLine* lines, double* gx,double* gy, GLine** ln1, GLine** ln2)
{
	if(lines==NULL){
		return 0;
	}
	int lncount=0;
	double a,b,c;
	if(lines->realp(&a,&b,&c)==false){
		return 0;
	}
	if(lines->next!=NULL){
		lncount = Snap2LineEx(lines->next,gx,gy,ln1,ln2);
	}
	double x0=-(a*c-b*(b*(*gx)-a*(*gy)))/(a*a+b*b);
	double y0=-(a*(b*(*gx)-a*(*gy))+b*c)/(a*a+b*b);
	if((x0- *gx)*(x0 - *gx)+(y0 - *gy)*(y0 - *gy) < snapradius2 && lines->invisible==false){
		if(lncount==0){
			*gx=x0;
			*gy=y0;
			*ln1=lines;
			*ln2=NULL;
			return 1;
		}
		else if(lncount==1){
			GPoint gp=Meet(lines,*ln1);
			*gx = gp.xz().real();
			*gy = gp.yz().real();
			*ln2=lines;
			return 2;
		}
	}
	return lncount;
}

bool MyCanvas::Snap2Circle(GCircle* circles, double* gx, double* gy)
{
	if(circles==NULL){
		return false;
	}
	double a,b,c;
	if(circles->realp(&a,&b,&c)==false){
		return false;
	}
	double r=sqrt((a - *gx)*(a - *gx)+(b - *gy)*(b - *gy));
	if( sqrt(c) - snapradius < r && r < sqrt(c) + snapradius && circles!=cursorci  && circles->invisible==false){
		Proj2Circle(circles,gx,gy);
		return true;
	}
	if(circles->next!=NULL){
		return Snap2Circle(circles->next,gx,gy);
	}
	return false;
}

int MyCanvas::Snap2CircleEx(GCircle* circles, double* gx,double* gy, GCircle** ci1, GCircle** ci2)
{
	if(circles==NULL){
		return 0;
	}
	int cicount=0;
	double a,b,c;
	if(circles->realp(&a,&b,&c)==false){
		return 0;
	}
	if(circles->next!=NULL){
		cicount = Snap2CircleEx(circles->next,gx,gy,ci1,ci2);
	}
	double r=sqrt((a - *gx)*(a - *gx)+(b - *gy)*(b - *gy));
	if( sqrt(c) - snapradius < r && r < sqrt(c) + snapradius && circles!=cursorci  && circles->invisible==false){
		if(cicount==0){
			Proj2Circle(circles,gx,gy);
			*ci1=circles;
			*ci2=NULL;
			return 1;
		}
		else if(cicount==1){
			Meet(circles,*ci1,gx,gy);
			*ci2=circles;
			return 2;
		}
	}
	return cicount;
}

void MyCanvas::PtDragHilight(GLine* ln, double gx,double gy)
{
	if(ln==NULL){
		return;
	}
	double a,b,c;
	if(ln->realp(&a,&b,&c)==false){
		ln->hilighted=false;
		return;
	}
	double x0=-(a*c-b*(b*gx-a*gy))/(a*a+b*b);
	double y0=-(a*(b*gx-a*gy)+b*c)/(a*a+b*b);
	if((x0- gx)*(x0 - gx)+(y0 - gy)*(y0 - gy) < snapradius2){
		ln->hilighted=true;
	}
	else if(!ShiftLButton) {
		ln->hilighted=false;
	}
	PtDragHilight(ln->next,gx,gy);
}

void MyCanvas::PtDragHilight(GCircle* ci, double gx,double gy)
{
	if(ci==NULL){
		return;
	}
	double a,b,c;
	if(ci->realp(&a,&b,&c)==false){
		ci->hilighted=false;
		return;
	}
	double r=sqrt((a - gx)*(a - gx)+(b - gy)*(b - gy));
	if( sqrt(c) - snapradius < r && r < sqrt(c) + snapradius){
		ci->hilighted=true;
	}
	else if(!ShiftLButton) {
		ci->hilighted=false;
	}
	PtDragHilight(ci->next,gx,gy);
}

void MyCanvas::PtDragHilight(GPoint* pt, double gx,double gy)
{
	if(pt==NULL){
		return;
	}
	double geomx=pt->xz().real();
	double geomy=pt->yz().real();
	if((geomx - gx)*(geomx - gx) + (geomy - gy)*(geomy - gy) < snapradius2){
		pt->hilighted=true;
	}
	else if(!ShiftLButton){
		pt->hilighted=false;
	}
	PtDragHilight(pt->next,gx,gy);
}

void MyCanvas::PtDragHilight(GParaCurve* cu, double gx,double gy)
{
	if(cu==NULL){
		return;
	}
	if(cu->orbit_len<=1){
		return;
	}
	double g1x,g1y,g2x,g2y;
	double x0,y0,t=0.0;
	bool snap=false;
	int i;
	g1x=cu->orbitx[0];
	g1y=cu->orbity[0];
	for(i=1;i<cu->orbit_len && !snap;i++){
		g2x=cu->orbitx[i];
		g2y=cu->orbity[i];
		if(g1x!=g2x || g1y!=g2y){
			t = ((gx-g1x)*(g2x-g1x)+(gy-g1y)*(g2y-g1y)) / ((g2x-g1x)*(g2x-g1x)+(g2y-g1y)*(g2y-g1y));
			x0 = g1x*(1-t)+g2x*t;
			y0 = g1y*(1-t)+g2y*t;
			if(t>=0.0 && t<1.0 && (x0- gx)*(x0 - gx)+(y0 - gy)*(y0 - gy) < snapradius2  && cu->invisible==false){
				snap=true;
			}
		}
		g1x=g2x;g1y=g2y;
	}
	if(snap){
		cu->hilighted=true;
	}
	else if(!ShiftLButton){
		cu->hilighted=false;
	}
	PtDragHilight(cu->next,gx,gy);
}

void MyCanvas::SelectHilight(GPoint* pt, double gx,double gy)
{
	if(pt==NULL){
		return;
	}
	double geomx=pt->xz().real();
	double geomy=pt->yz().real();
	if((geomx - gx)*(geomx - gx) + (geomy - gy)*(geomy - gy) < snapradius2 && pt->invisible==false){
		if(!ShiftLButton && lns!=NULL){
			lns->ClearHilight();
			NOhilightedLines=0;
		}
		if(!ShiftLButton && cis!=NULL){
			cis->ClearHilight();
			NOhilightedCircles=0;
		}
		if(!ShiftLButton && curves!=NULL){
			curves->ClearHilight();
			NOhilightedCurves=0;
		}
		if(!ShiftLButton && texts!=NULL){
			texts->ClearHilight();
		}
		Sethilighted(pt);
	}
	else if(!ShiftLButton){
		Deletehilighted(pt);
	}
	SelectHilight(pt->next,gx,gy);
}

void MyCanvas::SelectHilight(GLine* ln, double gx,double gy)
{
	if(ln==NULL){
		return;
	}
	double a,b,c;
	if(ln->realp(&a,&b,&c)==false){
		ln->hilighted=false;
		return;
	}
	double x0=-(a*c-b*(b*gx-a*gy))/(a*a+b*b);
	double y0=-(a*(b*gx-a*gy)+b*c)/(a*a+b*b);
	if((x0- gx)*(x0 - gx)+(y0 - gy)*(y0 - gy) < snapradius2){
		if(!ShiftLButton && pts!=NULL){
			pts->ClearHilight();
			NOhilightedPoints=0;
		}
		if(!ShiftLButton && cis!=NULL){
			cis->ClearHilight();
			NOhilightedCircles=0;
		}
		if(!ShiftLButton && curves!=NULL){
			curves->ClearHilight();
			NOhilightedCurves=0;
		}
		if(!ShiftLButton && texts!=NULL){
			texts->ClearHilight();
		}
		Sethilighted(ln);
		if(GMode==MODE_CINDY){
			ln->clipping=LineClipping;
		}
	}
	else if(!ShiftLButton){
		Deletehilighted(ln);
	}
	SelectHilight(ln->next,gx,gy);
}

void MyCanvas::SelectHilight(GCircle* ci, double gx,double gy)
{
	if(ci==NULL){
		return;
	}
	double a,b,c;
	if(ci->realp(&a,&b,&c)==false){
		ci->hilighted=false;
		return;
	}
	double r=sqrt((a - gx)*(a - gx)+(b - gy)*(b - gy));
	if( sqrt(c) - snapradius < r && r < sqrt(c) + snapradius){
		if(!ShiftLButton && pts!=NULL){
			pts->ClearHilight();
			NOhilightedPoints=0;
		}
		if(!ShiftLButton && lns!=NULL){
			lns->ClearHilight();
			NOhilightedLines=0;
		}
		if(!ShiftLButton && curves!=NULL){
			curves->ClearHilight();
			NOhilightedCurves=0;
		}
		if(!ShiftLButton && texts!=NULL){
			texts->ClearHilight();
		}
		Sethilighted(ci);
	}
	else if(!ShiftLButton){
		Deletehilighted(ci);
	}
	SelectHilight(ci->next,gx,gy);
}

bool MyCanvas::Snap2Text(GText* text, int px,int py)
{
	if(text==NULL){
		return false;
	}
	int length,height;
	wxPaintDC dc(this);
	PrepareDC(dc);
	wxFont* myfont = wxFont::New(text->fontsize, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD, false, wxT(""),  wxFONTENCODING_DEFAULT);
	dc.SetFont(*myfont);
	GetMultiTextSize((wxString)text->str,(wxMemoryDC*)&dc,&length,&height);
	delete myfont;
	if(text->tx<px && px<text->tx+length && text->ty<py && py<text->ty+height  && text->invisible==false){
		cursortext=text;
		cursortext->m_text_dragging_h_length=length/2;
		cursortext->m_text_dragging_h_height=height/2;
		return true;
	}
	else {
		cursortext=NULL;
		return Snap2Text(text->next,px,py);
	}
}

bool MyCanvas::Snap2Curve(GParaCurve* curve, double* gx,double *gy)
{
	if(curve==NULL){
		return false;
	}
	int i;
	if(curve->orbit_len<=1){
		return false;
	}
	double g1x,g1y,g2x,g2y;
	double x0,y0,t;
	g1x=curve->orbitx[0];
	g1y=curve->orbity[0];
	for(i=1;i<curve->orbit_len;i++){
		g2x=curve->orbitx[i];
		g2y=curve->orbity[i];
		if(g1x!=g2x || g1y!=g2y){
			t = ((*gx-g1x)*(g2x-g1x)+(*gy-g1y)*(g2y-g1y)) / ((g2x-g1x)*(g2x-g1x)+(g2y-g1y)*(g2y-g1y));
			x0 = g1x*(1-t)+g2x*t;
			y0 = g1y*(1-t)+g2y*t;
			if(t>=0.0 && t<1.0 && (x0- *gx)*(x0 - *gx)+(y0 - *gy)*(y0 - *gy) < snapradius2  && curve->invisible==false){
				*gx=x0;
				*gy=y0;
				return true;
			}
		}
		g1x=g2x;
		g1y=g2y;
	}
	if(curve->next!=NULL){
		return Snap2Curve(curve->next,gx,gy);
	}
	return false;
}

int MyCanvas::Snap2CurveEx(GParaCurve* cu, double* gx,double* gy, GParaCurve** cu1, GParaCurve** cu2)
{
	if(cu==NULL){
		return 0;
	}
	int cucount=0;
	if(cu->next!=NULL){
		cucount = Snap2CurveEx(cu->next,gx,gy,cu1,cu2);
		if(cucount>=2){
			return cucount;
		}
	}
	double g1x,g1y,g2x,g2y;
	double x0=0.0,y0=0.0,t=0.0;
	bool snap=false;
	int i;
	g1x=cu->orbitx[0];
	g1y=cu->orbity[0];
	for(i=1;i<cu->orbit_len && !snap;i++){
		g2x=cu->orbitx[i];
		g2y=cu->orbity[i];
		if(g1x!=g2x || g1y!=g2y){
			t = ((*gx-g1x)*(g2x-g1x)+(*gy-g1y)*(g2y-g1y)) / ((g2x-g1x)*(g2x-g1x)+(g2y-g1y)*(g2y-g1y));
			x0 = g1x*(1-t)+g2x*t;
			y0 = g1y*(1-t)+g2y*t;
			if(t>=0.0 && t<1.0 && (x0- *gx)*(x0 - *gx)+(y0 - *gy)*(y0 - *gy) < snapradius2  && cu->invisible==false){
				snap=true;
			}
		}
		g1x=g2x;g1y=g2y;
	}
	if(snap){
		if(cucount==0){
			*gx=x0;
			*gy=y0;
			*cu1=cu;
			*cu2=NULL;
			return 1;
		}
		else if(cucount==1){
			*gx=x0;
			*gy=y0;
			*cu2=cu;
			return 2;
		}
	}
	return cucount;
}

bool MyCanvas::Snap2PtName(GPoint* pt, int px,int py)
{
	if(pt==NULL){
		return false;
	}
	long length,height,descent;
	wxPaintDC dc(this);
	PrepareDC(dc);
	dc.SetFont(wxFont(pt->_fontsize,wxFONTFAMILY_DEFAULT,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL));
	dc.GetTextExtent(pt->_name,&length,&height,&descent);
	int tx = centerx + (int)floor((pt->xz().real()-geomcx)/geomperdot)+(int)floor((10+pt->_fontsize)*cos(pt->_name_theta))-(pt->_fontsize/2);
	int ty = centery - (int)floor((pt->yz().real()-geomcy)/geomperdot)-(int)floor((10+pt->_fontsize)*sin(pt->_name_theta))-(pt->_fontsize);
	if(tx<px && px<tx+length && ty<py && py<ty+height+descent && pt->invisible==false){
		cursorpt=pt;
		return true;
	}
	else {
		cursorpt=NULL;
		return Snap2PtName(pt->next,px,py);
	}
}

void MyCanvas::SelectHilight(GParaCurve* curve, double gx,double gy)
{
	if(curve==NULL){
		return;
	}
	bool good;
	int i;
	double g1x,g1y,g2x,g2y;
	double x0,y0,t;
	if(curve->orbit_len<=1){
		good=false;
	}
	else {
		good=false;
		g1x=curve->orbitx[0];
		g1y=curve->orbity[0];
		for(i=1;i<curve->orbit_len && good==false;i++){
			g2x=curve->orbitx[i];
			g2y=curve->orbity[i];
			if(g1x!=g2x || g1y!=g2y){
				t = ((gx-g1x)*(g2x-g1x)+(gy-g1y)*(g2y-g1y)) / ((g2x-g1x)*(g2x-g1x)+(g2y-g1y)*(g2y-g1y));
				x0 = g1x*(1-t)+g2x*t;
				y0 = g1y*(1-t)+g2y*t;
				if(t>=0.0 && t<1.0 && (x0- gx)*(x0 - gx)+(y0 - gy)*(y0 - gy) < snapradius2){
					good=true;
				}
			}
			g1x=g2x;
			g1y=g2y;
		}
	}
	if(good){
		if(!ShiftLButton && pts!=NULL){
			pts->ClearHilight();
			NOhilightedPoints=0;
		}
		if(!ShiftLButton && lns!=NULL){
			lns->ClearHilight();
			NOhilightedLines=0;
		}
		if(!ShiftLButton && cis!=NULL){
			cis->ClearHilight();
			NOhilightedCircles=0;
		}
		if(!ShiftLButton && curves!=NULL){
			curves->ClearHilight();
			NOhilightedCurves=0;
		}
		if(!ShiftLButton && texts!=NULL){
			texts->ClearHilight();
		}
		Sethilighted(curve);
	}
	else if(!ShiftLButton){
		Deletehilighted(curve);
	}
	SelectHilight(curve->next,gx,gy);
}

